import unittest, time, re
import sys
import math

# for chi2_contingency test, histograms
import numpy as np
from scipy import stats
import matplotlib.pyplot as plt
import matplotlib
from datetime import datetime, timedelta

# for G-test
import rpy2.robjects as robjects

# for Porter Stemming
from stemming.porter2 import stem

# for Octave
from oct2py import Oct2Py

# Permutation test
from itertools import combinations as comb
from itertools import permutations as perm

# Choices for what to uniquely identify an ad with
URL = 1
TITLE_URL = 2
TITLE_BODY = 3
CHOICE = URL

# Choices for measure of similarity
JACCARD = 1
COSINE = 2
SIM_CHOICE = COSINE

# Choices for assigning weight to the vector
NUM = 1
LOG_NUM = 2
SCALED_NUM = 3 # not implemented
W_CHOICE = LOG_NUM

CAT_FILE = "categories.txt"

#unittest.main(module = collector)

class Ad:

	def __init__(self, ad):
		self.title = ad['Title']
		self.url = ad['URL']
		self.body = ad['Body']
		self.cat = ad['cat']
		self.time = ad['Time']
	
	def ad_init(self, t, u, b, c, time):
		self.title = t
		self.url = u
		self.body = b
		self.cat = c
		self.time = time
	
	def display(self):
		print ("Title: "+self.title)
		print ("URL: "+self.url)
		print ("Body: "+self.body)
	
	def identical_ad(self, ad, choice):
		if(choice == URL):
			if(self.url == ad.url):
				return(True)
		elif(choice == TITLE_URL):
			if(self.url == ad.url and self.title == ad.title):
				return(True)
		elif(choice == TITLE_BODY):
			if(self.body == ad.body and self.title == ad.title):
				return(True)
		else:
			return(False)
	
	def set_from_cat_file(self, cat):
		for ca in cat.data:
			if(self.identical_ad(ca, TITLE_BODY)):
				self.categorize(ca.cat)
				sys.stdout.write('.')
				return True
		return False	
	
	def add_to_cat_file(self, ca, FILE):
		self.categorize(ca)
		t = (self.title + "||" + self.url + "||" + self.body.rstrip() + "||" + self.cat)
		fo = open(FILE, "a")
		fo.write(self.title + "||" + self.url + "||" + self.body.rstrip() + "||" + self.cat + '\n')
		fo.close()
		
		
	def categorize(self, ca):
		self.cat = ca

class AdVec:

	def __init__(self):
		self.data = []
	
	def index(self, ad):
		return self.data.index(ad)
		
	def size(self):
		return len(self.data)
	
	def add_vec(self, ads):
		for ad in ads.data:
			self.add(ad)
			
	def split_vec(self, index, len):
		temp = AdVec()
		for i in range(0,len):
			#print index+i
			temp.add(self.choose_by_index(index+i))
		return temp
	
	def split_time_vec(self, tmin, tmax):
		temp = AdVec()
		for ad in self.data:
			if(ad.time >= tmin and ad.time <= tmax):
				temp.add(ad)
		return temp
	
	def add(self, ad):
		self.data.append(ad)
		
	def remove(self, ad):
		self.data.remove(ad)
		
	def display(self, choice):
		print ("Total number unique of ads: "+str(len(self.data)))
		i = 0
		for ad in self.data:
			i += 1
			if(choice == 1):
				print (str(i)+" "+ad.url)
			if(choice == 2):
				print (str(i)+" "+ad.title)
			if(choice == 3):
				print (str(i)+" "+ad.body)

	def choose_by_index(self, index):
		return self.data[index]
	
	def get_cat_list(self):
		return [self.data[i].cat for i in range(0, len(self.data))]
	
	def purge_within_interval(self, interval):
		for ad in self.data:
			now = datetime.now()
			if(matplotlib.dates.date2num(now) > matplotlib.dates.date2num(ad.time) + interval):
				self.remove(ad)
		
	
	def read_ads_from_file(self, fname):
		fo = open(fname, "r")
		ads = []
		j = 0
		for line in fo:
			try:
				chunks = re.split("\|\|", line)
				a = Ad({'Time':datetime.strptime(chunks[0], "%Y-%m-%d %H:%M:%S.%f"), 'Title':chunks[1], 'URL': chunks[2], 'Body': chunks[3].rstrip(), 'cat': ""})
				self.add(a)
				#ads.append({'Title':chunks[0], 'URL': chunks[1], 'Body': chunks[2]})
				#ran.append(datetime.strptime(i, "%H:%M:%S.%f"))
				j += 1
			except:
				#i = line.rstrip()
				pass
		print ("Ads in "+fname+" = "+str(j))
		
	
	def read_ads_from_log(self, fname):
		fo = open(fname, "r")
		ads = []
		j = 0
		for line in fo:
			try:
				chunks = re.split("\|\|", line)
				a = Ad({'Time':datetime.strptime(chunks[0], "%Y-%m-%d %H:%M:%S.%f"), 'Title':chunks[1], 'URL': chunks[2], 'Body': chunks[3].rstrip(), 'cat': ""})
				self.add(a)
				#ads.append({'Title':chunks[0], 'URL': chunks[1], 'Body': chunks[2]})
				#ran.append(datetime.strptime(i, "%H:%M:%S.%f"))
				j += 1
			except:
				#i = line.rstrip()
				pass
		print ("Ads in "+fname+" = "+str(j))

	def same_ads(self, ad1, ad2):
		if(CHOICE == URL):
			if(ad1.url == ad2.url):
				return(True)
		elif(CHOICE == TITLE_URL):
			if(ad1.url == ad2.url and ad1.title == ad2.title):
				return(True)
		elif(CHOICE == TITLE_BODY):
			if(ad1.body == ad2.body and ad1.title == ad2.title):
				return(True)
		else:
			return(False)
			
	def unique_set(self, ads):
		for ad in ads.data:
			present = False
			for un_ad in self.data:
				if(self.same_ads(ad, un_ad)):
					present = True
					break					
			if(not present):
				self.add(ad)	
									
	def unique(self):
		uniq = AdVec()
		for ad in self.data:
			present = False
			for un_ad in uniq.data:
				if(self.same_ads(ad, un_ad)):
					present = True
					break					
			if(not present):
				uniq.add(ad)
		return uniq
			
	def union(self, ads1, ads2):
		temp = AdVec()
		for ad in ads1.data:
			temp.add(ad)
		for ad in ads2.data:
			temp.add(ad)
		self.unique_set(temp)
		return self
		
	def intersect(self, adsp1, adsp2):
		temp_int = AdVec()
		ads1 = AdVec()
		ads2 = AdVec()
		ads1.add_vec(adsp1)
		ads2.add_vec(adsp2)
		for ad1 in ads1.data:
			present = False
			for ad2 in ads2.data:
				if(self.same_ads(ad1, ad2)):
					present = True
					break
			if(present):
				temp_int.add(ad1)
				ads1.remove(ad1)
				ads2.remove(ad2)
		self.unique_set(temp_int)	
		return self	

	def tot_intersect(self, adsp1, adsp2):
		ads1 = AdVec()
		ads2 = AdVec()
		ads1.add_vec(adsp1)
		ads2.add_vec(adsp2)
		for ad1 in ads1.data:
			present = False
			for ad2 in ads2.data:
				if(self.same_ads(ad1, ad2)):
					present = True
					break
			if(present):
				self.add(ad1)
				ads2.remove(ad2)
		return self
					
	def ad_weight(self, ad):
		count = 0
		for a in self.data:
			if(self.same_ads(a, ad)):
				count = count+1
		if(W_CHOICE == NUM):
			return count
		elif(W_CHOICE == LOG_NUM):
			if(count == 0):
				return 0
			else:
				return math.log(count)
		else:
			print("Illegal W_CHOICE")
			sys.exit()
		
	def gen_temp_ad_vec(self, ads):		# self is ad_union
		vec = [0]*ads.size()
		i = 0
		j = 0
		for ad in ads.data:
			for lad in self.data:
				if(self.same_ads(ad, lad)):
					vec[i] = self.index(lad)
			i += 1
		return vec
		
	def gen_ad_vec(self, ads):			# self is ad_union
		vec = [0]*self.size()
		for ad in self.data:
			for lad in ads.data:
				if(self.same_ads(ad, lad)):
					vec[self.index(ad)] += 1
		return vec	
		
	def gen_cat_vec(self, ucats):		# self is ads
		vec = [0]*len(ucats)
		for ad in self.data:
			for cat in ucats:
				if(ad.cat == cat):
					vec[ucats.index(cat)] += 1
		return vec
	
	def gen_temp_cat_vec(self, ucats):	# self is ads
		vec = [0]*self.size()
		i = 0
		j = 0
		for ad in self.data:
			for cat in ucats:
				if(ad.cat == cat):
					vec[i] = ucats.index(cat)
			i += 1
		return vec
		
class Analyst:
	
#### GUHA'S ANALYSIS ####

# Ad Set Comparison

	def ad_sim(self, ads1, ads2):
		if(SIM_CHOICE == JACCARD):
			return self.jaccard_index(ads1, ads2)
		elif(SIM_CHOICE == COSINE):
			return self.ad_cosine_sim(ads1, ads2)
		else:
			print("Illegal SIM_CHOICE")
			sys.exit()

	def jaccard_index(self, ads1, ads2):
		ad_union = AdVec()
		ad_union.union(ads1, ads2)
# 		ad_union.add_vec(ads1)
# 		ad_union.add_vec(ads2)
		ad_int = AdVec()
		ad_int.intersect(ads1, ads2)
# 		ad_int.tot_intersect(ads1, ads2)
		return (float(ad_int.size())/float(ad_union.size()))
	
	def ad_cosine_sim(self, ads1, ads2):
		ad_union = AdVec()
		ad_union.union(ads1, ads2)
		vec1 = []
		vec2 = []
		for ad in ad_union.data:
			vec1.append(ads1.ad_weight(ad))
			vec2.append(ads2.ad_weight(ad))
		return self.cosine_sim(vec1, vec2)
	
# Vector Operations

	def cosine_sim(self, vec1, vec2):
		return (self.dot_prod(vec1, vec2)/(self.vec_mag(vec1)*self.vec_mag(vec2)))

	def vec_mag(self, vec):
		sum = 0.0
		for i in vec:
			sum = sum + i*i
		return math.sqrt(sum)
	
	def dot_prod(self, vec1, vec2):
		sum = 0.0
		if(len(vec1) != len(vec2)):
			print("Dot product doesnt exist")
			sys.exit()
		for i in range(0, len(vec1)):
			sum = sum + vec1[i]*vec2[i]
		return sum

#### MY ANALYSES ####
	
	def temp_ad_vectors(self, list):
		ad_union = AdVec()
		for ads in list:
			ad_union.union(ads, ad_union)
		ad_union.display(2)
		tav_list = []
		for ads in list:
			tav_list.append(ad_union.gen_temp_ad_vec(ads))
		return tav_list

	def word_vectors(self, list):
		ad_union = AdVec()
		for ads in list:
			ad_union.union(ads, ad_union)
		words = self.ads_to_words(ad_union)
		stemmed_words = self.stem_vec(words)
		word_v = self.unique_words(stemmed_words)
		word_v = self.strip_vec(word_v)
# 		for asd in word_v:
# 			print str(word_v.index(asd))+" "+asd
		wv_list = []
		for ads in list:
			wv_list.append(self.gen_vec(ads, word_v))
		return wv_list
		
	def ad_vectors(self, list):
		ad_union = AdVec()
		for ads in list:
			ad_union.union(ads, ad_union)
		ad_union.display(2)
		av_list = []
		for ads in list:
			av_list.append(ad_union.gen_ad_vec(ads))
		return av_list

	def ad_category(self, list):
		cats = []
		for ads in list:
			cats.extend(ads.get_cat_list())
		ucats = self.unique_words(cats)
		ucats.sort()
		print ""
		for c in ucats:
			print (str(ucats.index(c))+" "+c)
		ac_list = []
		for ads in list:
			ac_list.append(ads.gen_cat_vec(ucats))
		ac_list.append(ucats)
		return ac_list

	def ad_temp_category(self, list):
		cats = []
		for ads in list:
			cats.extend(ads.get_cat_list())
		ucats = self.unique_words(cats)
		ucats.sort()
		print ""
		for c in ucats:
			print (str(ucats.index(c))+" "+c)
		ac_list = []
		for ads in list:
			ac_list.append(ads.gen_temp_cat_vec(ucats))
		ac_list.append(ucats)
		return ac_list
	
# Word based analysis

		
	def ads_to_words(self, ads):
		line = ""
		for ad in ads.data:
			line = line + " " + ad.title+ " " + ad.body
		return re.split(r'[.(), !?;\-\n]+', line)
	
	def unique_words(self, words):
		unq = []
		for word in words:
			present = False
			for un in unq:
				if (un == word):
					present = True
					break
			if(not present):
				unq.append(word)
		return unq
	
	def stem_vec(self, words):
		for i in range(0, len(words)-1):
			words[i] = stem(words[i]).lower()
		return words
	
	def strip_vec(self, list):
		if(list[0] == ''):
			del list[-len(list)]
		if(list[len(list)-1] == ''):
			del list[-1]
		chars = set('0123456789$&')
		return [x for x in list if not (any((c in chars) for c in x))]
	
	def gen_vec(self, ads, word_v):
		vec = []
		words = self.ads_to_words(ads)
		stemmed_words = self.stem_vec(words)
		words = self.strip_vec(words)
		# print words
		for word in word_v:
			vec.append(words.count(word))	# set weight of the vectors according to some measure, now its simple count
		# print vec
		# print ("Length = "+str(len(vec)))
		# print ("Number of 0s = "+str(sum(x == 0 for x in vec)))
		return vec
			
# Category based analysis

	def read_categories(self, FILE):
		fo = open(FILE, 'r')
		cats = AdVec()
		for line in fo:
			chunks = re.split("\|\|", line)
			ad = Ad({'Title':chunks[0], 'URL': chunks[1], 'Body': chunks[2], 'cat':chunks[3].rstrip()})
			cats.add(ad)
		return cats		

	def set_categories(self, ads, cat):	
		cats = cat.get_cat_list()
		ucats = self.unique_words(cats)
		ucats.sort()
		i = 0
		for ad in ads.data:
			i += 1
			if( not ad.set_from_cat_file(cat)):
				print('\n')
				ad.display()
				print ("\nExisting Categories:")
				print ucats
				while(True):
					entry = raw_input("Enter category: ")
					con = raw_input("Did you enter "+entry+"? [y/n]")
					if con == 'y':
						break
					else:
						print ("Lets try again...")
				ad.add_to_cat_file(entry, CAT_FILE)
				cat.add(ad)
				cats = cat.get_cat_list()
				ucats = self.unique_words(cats)
				ucats.sort()
				print (str(len(ads.data)-i)+" remaining..")

		
# Cosine Similarity on a graph

# 	def plot_ad_sim(self, ads1, ads2, typ):
# 		l = []
# 		c = []
# 		t = min(ads1.size(), ads2.size())
# 		print t
# 		for i in range(1,t):
# 			#print i
# 			c.append(self.ad_sim(ads1.split_vec(0, i), ads2.split_vec(0, i)))
# 			l.append(i)
# 		plt.plot(l, c, typ, alpha=1, linestyle='-', label = 'cosine')
# 		
	def plot_ad_sim(self, ads1, ads2, typ, lbl):
		l = []
		c = []
		#t_min = min(ads1.choose_by_index(0).time, ads2.choose_by_index(0).time)
		#t_max = max(ads1.choose_by_index(ads1.size()-1).time, ads2.choose_by_index(ads2.size()-1).time)
		t_min = []
		t_max = []
		t_min.append(min(ads1.choose_by_index(0).time, ads2.choose_by_index(0).time))
		t_max.append(max(ads1.choose_by_index(ads1.size()-1).time, ads2.choose_by_index(ads2.size()-1).time))

# 		t_min.append(datetime.strptime("2013-10-31 22:22:44.646522", "%Y-%m-%d %H:%M:%S.%f"))
# 		t_max.append(datetime.strptime("2013-11-02 18:30:13.354417", "%Y-%m-%d %H:%M:%S.%f"))
# 		t_min.append(datetime.strptime("2013-11-01 05:26:28.971169", "%Y-%m-%d %H:%M:%S.%f"))
# 		t_max.append(datetime.strptime("2013-11-01 07:13:57.749954", "%Y-%m-%d %H:%M:%S.%f"))
# 		t_min.append(datetime.strptime("2013-11-01 12:33:36.236596", "%Y-%m-%d %H:%M:%S.%f"))
# 		t_max.append(datetime.strptime("2013-11-01 14:07:39.290593", "%Y-%m-%d %H:%M:%S.%f"))
# 		t_min.append(datetime.strptime("2013-11-01 19:28:34.924864", "%Y-%m-%d %H:%M:%S.%f"))
# 		t_max.append(datetime.strptime("2013-11-01 21:06:23.174660", "%Y-%m-%d %H:%M:%S.%f"))
# 		t_min.append(datetime.strptime("2013-11-02 02:41:27.219158", "%Y-%m-%d %H:%M:%S.%f")) #2became F, 3 became M
# 		t_max.append(datetime.strptime("2013-11-02 04:03:39.070038", "%Y-%m-%d %H:%M:%S.%f"))
# 		t_min.append(datetime.strptime("2013-11-02 04:35:32.496559", "%Y-%m-%d %H:%M:%S.%f")) # only 1, 3 survive 
# 		t_max.append(datetime.strptime("2013-11-02 16:14:16.244301", "%Y-%m-%d %H:%M:%S.%f"))
# 		
		j=0
		for i in range(0, len(t_min)):
			print t_min[i]
			print t_max[i]
			temp = t_min[i]
			while temp <= t_max[i]:
				temp = temp+timedelta(minutes=2)
				j += 1
				try:
					c.append(self.ad_sim(ads1.split_time_vec(t_min[i], temp), ads2.split_time_vec(t_min[i], temp)))
					l.append(j)
				except:
					pass
			c.append(0)
			l.append(j+1)
			j +=50
			c.append(0)
			l.append(j)
		plt.plot(l, c, typ, alpha=1, linestyle='-', label = lbl)
		
		
	def statistic(self, a, b):
		return (b.unique().size() - a.unique().size())
		#return -self.ad_sim(a, b)
	#     sumab, suma = sum(ab), sum(a)
	#     return ( suma / len(a) -
	#              (sumab -suma) / (len(ab) - len(a)) )
 
	def permutationTest(self, adv, ass):
		advm = AdVec()
		advf = AdVec()
		print ass
		for i in range(0, len(adv)):
			if(i in ass[0:len(ass)/2]):
				print "c"+str(i)
				advm.add_vec(adv[i])
			else:
				print "t"+str(i)
				advf.add_vec(adv[i])
# 		advm.display(2)
# 		advf.display(2)
		Tobs = self.statistic(advm, advf)
		#print Tobs
		under = 0
		org_ass = ass
		for count, per in enumerate(comb(ass, len(ass)/2), 1):
			#print count
			#print per
			advm = AdVec()
			advf = AdVec()
			for i in range(0, len(adv)):
				if(i in per[0:len(ass)/2]):
					#print "c"+str(i)
					advm.add_vec(adv[i])
				else:
					#print "t"+str(i)
					advf.add_vec(adv[i])
			#print self.statistic(advm, advf)
			#raw_input("wait")
			if self.statistic(advm, advf) >= Tobs:
				under += 1
		return under * 100. / count



if __name__ == "__main__":
	INSTANCES = 8
	SAMPLES = 20
	a = Analyst()
	inter = AdVec()
	adv = []
	for i in range(0, INSTANCES):
 		adv.append(AdVec())
 	
	advstar = []
	for i in range(0, SAMPLES):
 		advstar.append(AdVec())
 
	advm = AdVec()
	advf = AdVec()

	fo = open("log20-2", "r")
	r = 0
	m=0
	f=0
	sample_ass = []
	for line in fo:
		chunks = re.split("\|\|", line)
		chunks[len(chunks)-1] = chunks[len(chunks)-1].rstrip()
 		if(chunks[0] == 's' ):
 			continue		
#  		if(chunks[0] == 'g' and r >5 ):
#  			break
 		if(chunks[0] == 'g' and r >0 ):
#  			if(check == True):
#  				sample_ass.append(1)
#  			else:
#   				sample_ass.append(2)
			if(adv[0].size() > 0 and int(ass[0]) != 0):
# 				print ass[0]
# 				print r-1
# 				raw_input("wait")
				advstar[r-1] = adv[int(ass[0])]
				advm.add_vec(adv[int(ass[0])])
				print "m"
				m +=1
			elif(adv[1].size() > 0 and int(ass[0]) == 0):
# 				print ass[0]
# 				print r-1
# 				raw_input("wait")
				advstar[r-1] = adv[int(ass[0])]
				advm.add_vec(adv[int(ass[0])])
				print "m"
				m +=1
			else:
# 				print ass[0]
# 				print r-1
				advstar[r-1] = adv[int(ass[0])]
				advf.add_vec(adv[int(ass[0])])
				#advstar[r-1].display(2)
				print "f"
# 				f +=1
#  			under = a.permutationTest(adv, ass)
# 			print("under=%.2f%%, over=%.2f%%" % (under, 100. - under))	
			ass = chunks[1:]
			adv = []
			for i in range(0, INSTANCES):
 				adv.append(AdVec())
 			r += 1	
 			check = False
			#print ass
		elif(chunks[0] == 'g' and r==0):
			r += 1
			ass = chunks[1:]
			check = False
			#print ass
		else:
 			try:
				ad = Ad({'Time':datetime.strptime(chunks[1], "%Y-%m-%d %H:%M:%S.%f"), 'Title':chunks[2], 'URL': chunks[3], 'Body': chunks[4].rstrip(), 'cat': ""})
				#ad.display()
				#print chunks[0]
				#raw_input("wait")
				adv[int(chunks[0])].add(ad)
				
# 				if(ass[0] != chunks[0]):
# 					check = True
# 				else:
# 					advstar[r-1].add(ad)
			except:
				pass
	

	if(adv[0].size() > 0 and int(ass[0]) != 0):
		advstar[r-1] = adv[int(ass[0])]
		advm.add_vec(adv[int(ass[0])])
		print "m"
		m +=1
	elif(adv[1].size() > 0 and int(ass[0]) == 0):
		advstar[r-1] = adv[int(ass[0])]
		advm.add_vec(adv[int(ass[0])])
		print "m"
		m +=1
	else:
		advstar[r-1] = adv[int(ass[0])]
		advf.add_vec(adv[int(ass[0])])
		print "f"
		f +=1				
	
	print advm.unique().size()
	print advf.unique().size()
	
	sys.exit()
	
 	sample_ass = [8,3,2,0,4,1,6,7,5,9]
 	#[5,4,7,6,0,2,1,3]#[2, 3, 5, 7, 8, 0, 1, 4, 6, 9]#[14,5,11,17,6,9,12,3,10,2,13,16,0,19,1,18,15,7,4,8]
 	#[2, 3, 5, 7, 8, 0, 1, 4, 6, 9] #[5,4,7,6,0,2,1,3]
 	
 	s = datetime.now()
 	print s
 	#under = a.permutationTest(advstar, sample_ass)
 	print("under=%.2f%%, over=%.2f%%" % (under, 100. - under))
	e = datetime.now()
	print e
	print (e - s)
	
	sys.exit()
	
	advm.display(2)
	advf.display(2)
	
	list = [advm, advf]
	
# 	sys.exit()
# 	 
# 	under = a.permutationTest(adv, ass)
# 	print("under=%.2f%%, over=%.2f%%" % (under, 100. - under))
# 	sys.exit()
# 	print m
# 	print f
	
	#--perm test on adv[]
# 	list = [advm, advf]
# 	advm.display(2)
# 	advf.display(2)
# 	sys.exit()
	print ass

# 
# 	under = a.permutationTest(adv, ass)
# 	print("under=%.2f%%, over=%.2f%%" % (under, 100. - under))

	
# 	for line in fo:
# 		chunks = re.split("\|\|", line)
# 		chunks[len(chunks)-1] = chunks[len(chunks)-1].rstrip()
# 		if(chunks[0] == 'g'):
# 			ass = chunks[1:]
# 		else:
# 			try:
# 				ad = Ad({'Time':datetime.strptime(chunks[1], "%Y-%m-%d %H:%M:%S.%f"), 'Title':chunks[2], 'URL': chunks[3], 'Body': chunks[4].rstrip(), 'cat': ""})
# 				if(chunks[0] in ass[0:len(ass)/2]):
# 					advm.add(ad)
# 				else:
# 					advf.add(ad)
# 			except:
# 				pass


# 	treatmentGroup = obs[0]
# 	controlGroup   = obs[1]
# 	under = permutationTest(treatmentGroup, controlGroup)
# 	print("under=%.2f%%, over=%.2f%%" % (under, 100. - under))
	
# 	fo = open("log_org", "r")
# 	c=[]
# 	l=[]
# 	j=0
# 	for line in fo:
# 		chunks = re.split("\|\|", line)
# 		chunks[len(chunks)-1] = chunks[len(chunks)-1].rstrip()
# 		if(chunks[0] == 'g'):
# 			ass = chunks[1:]
# 			print ass
# 			try:
# 				c.append(a.ad_sim(advm, advf))
# 				l.append(j)
# 				j += 1
# 			except:
# 				pass
# 			advm = AdVec()
# 			advf = AdVec()
# 		else:
# 			try:
# 				ad = Ad({'Time':datetime.strptime(chunks[1], "%Y-%m-%d %H:%M:%S.%f"), 'Title':chunks[2], 'URL': chunks[3], 'Body': chunks[4].rstrip(), 'cat': ""})
# 				if(chunks[0] in ass[0:len(ass)/2]):
# 					advm.add(ad)
# 				else:
# 					advf.add(ad)
# 			except:
# 				pass
# 	
# # 		advm.display(2)
# # 		advf.display(2)
# # 		print j
# 	
# 	gridLineWidth=0.1
# 	fig, ax = plt.subplots()
# 	ax.xaxis.grid(True, zorder=0)
# 	ax.yaxis.grid(True, zorder=0)
#    	a.plot_ad_sim(advm, advf, 'm.', "mf")
#    	#plt.plot(l, c, "m.", alpha=0.5)
#  	plt.legend()
#  	plt.show()

		
# 	t = fo.readline().rstrip()
# 	
# 	print t
# 	chunks = re.split("\|\|", t)
# 	print chunks
	
	
# 	for i in range(0, INSTANCES):
# 		adv[i].read_ads_from_file("ads"+chunks[i]+".txt")
# 
# 	sys.exit()
# 	gridLineWidth=0.1
# 	fig, ax = plt.subplots()
# 	ax.xaxis.grid(True, zorder=0)
# 	ax.yaxis.grid(True, zorder=0)
#   	a.plot_ad_sim(adv1, adv2, 'g.', "12")
#   	a.plot_ad_sim(adv1, adv4, 'g.', "14")
#   	a.plot_ad_sim(adv4, adv3, 'm.', "43")
#   	a.plot_ad_sim(adv2, adv3, 'm.', "23")
#   	a.plot_ad_sim(adv1, adv3, 'b.', "13")
#   	a.plot_ad_sim(adv2, adv4, 'r.', "24")
# 	plt.legend()
# 	plt.show()
# 	sys.exit()

# 	print ("13= " +str(a.ad_sim(adv1, adv3)))
# 	print ("24= " +str(a.ad_sim(adv2, adv4)))
#	print ("12= " +str(a.ad_sim(adv1, adv2)))
# 	print ("14= " +str(a.ad_sim(adv1, adv4)))
# 	print ("34= " +str(a.ad_sim(adv3, adv4)))
# 	print ("32= " +str(a.ad_sim(adv3, adv2)))
	
# 	cat = a.read_categories(CAT_FILE)
# 	a.set_categories(adv1, cat)
# 	a.set_categories(adv2, cat)
	
	#list = [adv1, adv2, adv3, adv4]
	
	#obs = np.array(a.ad_category(list))
	#obs = np.array(a.ad_temp_category(list))
	#obs = np.array(a.temp_ad_vectors(list))
	obs = np.array(a.ad_vectors(list))
	#obs = np.array(a.word_vectors(list))
		
# 	print("Unique in ad1= "+ str(adv1.unique().size()))
# 	#a.print_ads(a.unique_set(ads1), 1)
# 	print("Unique in ad2= "+ str(adv2.unique().size()))
# 	#a.print_ads(a.unique_set(ads2), 1)
# 	print("Unique in ad3= "+ str(adv3.unique().size()))
# 	#a.print_ads(a.unique_set(ads1), 1)
# 	print("Unique in ad4= "+ str(adv4.unique().size()))
# 	#a.print_ads(a.unique_set(ads2), 1)
# 	print("Unique Intersection= " +str(inter.intersect(adv1, adv2).size()))
# 	#a.print_ads(a.intersect(ads1, ads2), 1)
# 	print("Total in ad1= "+ str(adv1.size()))
# 	print("Total in ad2= "+ str(adv2.size()))
# 	inter = AdVec()
# 	print("Total Intersection= " +str(inter.tot_intersect(adv1, adv2).size()))
# 	#print a.tot_intersect(ads1, ads2)


	
#### Statistical measures
	
# 	chi2, p, dof, ex = stats.chi2_contingency(obs)
# 	print ("Chi-Square = "+str(chi2))
# 	print ("p-value = "+str(p))
# 	print ("Similarity = "+str(a.ad_sim(ads1, ads2)))
# 	
# 	robjects.r.source('gtest.R')
# 	l = [val for pair in zip(obs[0], obs[1]) for val in pair]
# 	v = robjects.IntVector(l)
# 	m = robjects.r['matrix'](v, nrow = 2)
# 	#print(m)
# 	print("Size of ads:: "+str(len(ads1))+" -- "+str(len(ads2)))
# 	gt = robjects.r['g.test']
# 	ex, dof, psi, gval, pval = gt(m)
# 	print ("Psi-value = "+str(psi[0]))
# 	print ("G-value = "+str(gval[0]))
# 	print ("p-value = "+str(pval[0]))
	
#### Octave Bindings
	
# 	oc = Oct2Py()
# 	oc.plot(obs[0], '-o')
	
#### Temporal Plots

# 	print obs[0]
# 
# 	date1 = matplotlib.dates.date2num([adv1.data[i].time for i in range(0, len(adv1.data))])
# 	date2 = matplotlib.dates.date2num([adv2.data[i].time for i in range(0, len(adv2.data))])
# 	date3 = matplotlib.dates.date2num([adv3.data[i].time for i in range(0, len(adv3.data))])
# 	date4 = matplotlib.dates.date2num([adv4.data[i].time for i in range(0, len(adv4.data))])
# 	pos = np.arange(len(obs[1]))
# 	gridLineWidth=0.1
# 	fig, ax = plt.subplots()
# 	ax.xaxis.grid(True, zorder=0)
# 	ax.yaxis.grid(True, zorder=0)
# 	plt.plot(obs[0], date1, 'r.', alpha=0.5, label = 'ads1')
#  	plt.plot(obs[1], date2, 'g.', alpha=0.5, label = 'ads2')
#  	plt.plot(obs[2], date3, 'b.', alpha=0.5, label = 'ads3')
#  	plt.plot(obs[3], date4, 'm.', alpha=0.5, label = 'ads4')
# 	#plt.xticks(pos, obs[2], rotation='vertical')
# 	#plt.axis([-1, 500, 0, 700])
# 	plt.legend()
# 	plt.show()

# 	gridLineWidth=0.1
# 	fig, ax = plt.subplots()
# 	ax.xaxis.grid(True, zorder=0)
# 	ax.yaxis.grid(True, zorder=0)
# 	plt.plot(obs[0], ran1, 'r.', alpha=0.5, label = 'ads1')
# 	plt.plot(obs[1], ran2, 'g.', alpha=0.5, label = 'ads2')
# 	#plt.xticks(pos, obs[2], rotation='vertical')
# 	#plt.axis([-1, len(obs[2]), 0, len(ran1)/2+10])
# 	plt.legend()
# 	plt.show()
# 	gridLineWidth=0.1
# 	fig, ax = plt.subplots()
# 	ax.xaxis.grid(True, zorder=0)
# 	ax.yaxis.grid(True, zorder=0)
# 	plt.plot(obs[1], ran2, 'g.', alpha=0.5, label = 'ads2')
#  	plt.plot(obs[2], ran3, 'b.', alpha=0.5, label = 'ads3')
# 	#plt.xticks(pos, obs[2], rotation='vertical')
# 	#plt.axis([-1, len(obs[2]), 0, len(ran1)/2+10])
# 	plt.legend()
# 	plt.show()
# 	gridLineWidth=0.1
# 	fig, ax = plt.subplots()
# 	ax.xaxis.grid(True, zorder=0)
# 	ax.yaxis.grid(True, zorder=0)
#  	plt.plot(obs[2], ran3, 'b.', alpha=0.5, label = 'ads3')
#  	plt.plot(obs[3], ran4, 'm.', alpha=0.5, label = 'ads4')
# 	#plt.xticks(pos, obs[2], rotation='vertical')
# 	#plt.axis([-1, len(obs[2]), 0, len(ran1)/2+10])
# 	plt.legend()
# 	plt.show()
# 	gridLineWidth=0.1
# 	fig, ax = plt.subplots()
# 	ax.xaxis.grid(True, zorder=0)
# 	ax.yaxis.grid(True, zorder=0)
# 	plt.plot(obs[0], ran1, 'r.', alpha=0.5, label = 'ads1')
#  	plt.plot(obs[3], ran4, 'm.', alpha=0.5, label = 'ads4')
# 	#plt.xticks(pos, obs[2], rotation='vertical')
# 	#plt.axis([-1, len(obs[2]), 0, len(ran1)/2+10])
# 	plt.legend()
# 	plt.show()
# 	gridLineWidth=0.1
# 	fig, ax = plt.subplots()
# 	ax.xaxis.grid(True, zorder=0)
# 	ax.yaxis.grid(True, zorder=0)
# 	plt.plot(obs[1], ran2, 'g.', alpha=0.5, label = 'ads2')
#  	plt.plot(obs[3], ran4, 'm.', alpha=0.5, label = 'ads4')
# 	#plt.xticks(pos, obs[2], rotation='vertical')
# 	#plt.axis([-1, len(obs[2]), 0, len(ran1)/2+10])
# 	plt.legend()
# 	plt.show()
# 	gridLineWidth=0.1
# 	fig, ax = plt.subplots()
# 	ax.xaxis.grid(True, zorder=0)
# 	ax.yaxis.grid(True, zorder=0)
# 	plt.plot(obs[0], ran1, 'r.', alpha=0.5, label = 'ads1')
#  	plt.plot(obs[2], ran3, 'b.', alpha=0.5, label = 'ads3')
# 	#plt.xticks(pos, obs[2], rotation='vertical')
# 	#plt.axis([-1, len(obs[2]), 0, len(ran1)/2+10])
# 	plt.legend()
# 	plt.show()

#### Temporal Categories

# 	pos = np.arange(len(obs[2]))
#	width = 0.5     # gives histogram aspect to the bar diagram
# 	gridLineWidth=0.1
# 	fig, ax = plt.subplots()
# 	ax.xaxis.grid(True, zorder=0)
# 	ax.yaxis.grid(True, zorder=0)
# 	plt.plot(obs[0], ran1, 'r.', alpha=0.5, label = 'ads1')
# 	plt.plot(obs[1], ran2, 'g.', alpha=0.5, label = 'ads2')
# 	plt.plot(obs[2], ran3, 'b.', alpha=0.5, label = 'ads3')
# 	plt.plot(obs[3], ran4, 'm.', alpha=0.5, label = 'ads4')
# 	plt.xticks(pos+width/2., obs[2], rotation='vertical')
# 	plt.axis([-1, len(obs[2]), 0, len(ran1)/2+10])
# 	plt.legend()
# 	plt.show()
	
#### Histogram plots
	
	l1 = [int(i) for i in obs[0]]
	l2 = [int(i) for i in obs[1]]
# 	l3 = [int(i) for i in obs[2]]
# 	l4 = [int(i) for i in obs[3]]
	print obs
	pos = np.arange(len(obs[1]))
	width = 0.5     # gives histogram aspect to the bar diagram
 	gridLineWidth=0.1
 	fig, ax = plt.subplots()
 	ax.xaxis.grid(True, zorder=0)
 	ax.yaxis.grid(True, zorder=0)
	#plt.text(160, 100, r'P1-red, P2-green')
	plt.bar(pos, l1, width, color='r', alpha=0.5, label = 'friendly')
	plt.bar(pos, l2, width, color='g', alpha=0.5, label = 'lonely')
# 	plt.bar(pos, l3, width, color='b', alpha=0.5, label = 'ads3')
# 	plt.bar(pos, l4, width, color='m', alpha=0.5, label = 'ads4')
# 	plt.plot(obs[0], date1, 'r.', alpha=0.5, label = 'ads1')
#  	plt.plot(obs[1], date2, 'g.', alpha=0.5, label = 'ads2')
#  	plt.plot(obs[2], date3, 'b.', alpha=0.5, label = 'ads3')
#  	plt.plot(obs[3], date4, 'm.', alpha=0.5, label = 'ads4')
# 	plt.plot(pos, l1, color='r', alpha=0.5, label = 'ads1')
# 	plt.plot(pos, l2, color='g', alpha=0.5, label = 'ads2')
# 	plt.plot(pos, l3, color='b', alpha=0.5, label = 'ads3')
# 	plt.plot(pos, l4, color='m', alpha=0.5, label = 'ads4')
	#plt.xticks(pos+width/2., obs[2], rotation='vertical')
	#plt.axis([-1, len(obs[2]), 0, len(ran1)/2+10])
	plt.legend()
	plt.show()