import unittest, time, re
import sys
import math

# for chi2_contingency test, histograms
import numpy as np
from scipy import stats
import matplotlib.pyplot as plt

# for G-test
import rpy2.robjects as robjects

# for Porter Stemming
from stemming.porter2 import stem

# for Octave
from oct2py import Oct2Py

# Choices for what to uniquely identify an ad with
URL = 1
TITLE_URL = 2
TITLE_BODY = 3
CHOICE = URL

# Choices for measure of similarity
JACCARD = 1
COSINE = 2
SIM_CHOICE = JACCARD

# Choices for assigning weight to the vector
NUM = 1
LOG_NUM = 2
SCALED_NUM = 3 # not implemented
W_CHOICE = NUM

CAT_FILE = "categories.txt"

#unittest.main(module = collector)

class Ad:

	def __init__(self, ad):
		self.title = ad['Title']
		self.url = ad['URL']
		self.body = ad['Body']
		self.cat = ad['cat']
	
	def ad_init(self, t, u, b, c):
		self.title = t
		self.url = u
		self.body = b
		self.cat = c
	
	def display(self):
		print ("Title: "+self.title)
		print ("URL: "+self.url)
		print ("Body: "+self.body)
	
	def identical_ad(self, ad, choice):
		if(choice == URL):
			if(self.url == ad.url):
				return(True)
		elif(choice == TITLE_URL):
			if(self.url == ad.url and self.title == ad.title):
				return(True)
		elif(choice == TITLE_BODY):
			if(self.body == ad.body and self.title == ad.title):
				return(True)
		else:
			return(False)
	
	def set_from_cat_file(self, cat):
		for ca in cat.data:
			if(self.identical_ad(ca, TITLE_BODY)):
				self.categorize(ca.cat)
				sys.stdout.write('.')
				return True
		return False	
	
	def add_to_cat_file(self, ca, FILE):
		self.categorize(ca)
		t = (self.title + "||" + self.url + "||" + self.body.rstrip() + "||" + self.cat)
		fo = open(FILE, "a")
		fo.write(self.title + "||" + self.url + "||" + self.body.rstrip() + "||" + self.cat + '\n')
		fo.close()
		
		
	def categorize(self, ca):
		self.cat = ca

class AdVec:

	def __init__(self):
		self.data = []
	
	def index(self, ad):
		return self.data.index(ad)
		
	def size(self):
		return len(self.data)
	
	def add_vec(self, ads):
		for ad in ads.data:
			self.add(ad)
	
	def add(self, ad):
		self.data.append(ad)
		
	def remove(self, ad):
		self.data.remove(ad)
		
	def display(self, choice):
		print ("Number of ads: "+str(len(self.data)))
		i = 0
		for ad in self.data:
			i += 1
			if(choice == 1):
				print (str(i)+" "+ad.url)
			if(choice == 2):
				print (str(i)+" "+ad.title)
			if(choice == 3):
				print (str(i)+" "+ad.body)

	def choose_by_index(self, index):
		return self.data[index]
	
	def get_cat_list(self):
		return [self.data[i].cat for i in range(0, len(self.data))]
	
	def read_ads_from_file(self, fname, ran):
		fo = open(fname, "r")
		ads = []
		i = 0
		for line in fo:
			try:
				chunks = re.split("\|\|", line)
				a = Ad({'Title':chunks[0], 'URL': chunks[1], 'Body': chunks[2].rstrip(), 'cat': ""})
				self.add(a)
				#ads.append({'Title':chunks[0], 'URL': chunks[1], 'Body': chunks[2]})
				ran.append(i)
			except:
				i += 1
				pass

	def same_ads(self, ad1, ad2):
		if(CHOICE == URL):
			if(ad1.url == ad2.url):
				return(True)
		elif(CHOICE == TITLE_URL):
			if(ad1.url == ad2.url and ad1.title == ad2.title):
				return(True)
		elif(CHOICE == TITLE_BODY):
			if(ad1.body == ad2.body and ad1.title == ad2.title):
				return(True)
		else:
			return(False)
			
	def unique_set(self, ads):
		for ad in ads.data:
			present = False
			for un_ad in self.data:
				if(self.same_ads(ad, un_ad)):
					present = True
					break					
			if(not present):
				self.add(ad)	
						
	def unique(self):
		uniq = AdVec()
		for ad in self.data:
			present = False
			for un_ad in uniq.data:
				if(self.same_ads(ad, un_ad)):
					present = True
					break					
			if(not present):
				uniq.add(ad)
		return uniq
			
	def union(self, ads1, ads2):
		temp = AdVec()
		for ad in ads1.data:
			temp.add(ad)
		for ad in ads2.data:
			temp.add(ad)
		self.unique_set(temp)
		return self
		
	def intersect(self, adsp1, adsp2):
		temp_int = AdVec()
		ads1 = AdVec()
		ads2 = AdVec()
		ads1.add_vec(adsp1)
		ads2.add_vec(adsp2)
		for ad1 in ads1.data:
			present = False
			for ad2 in ads2.data:
				if(self.same_ads(ad1, ad2)):
					present = True
					break
			if(present):
				temp_int.add(ad1)
				ads1.remove(ad1)
				ads2.remove(ad2)
		self.unique_set(temp_int)	
		return self	

	def tot_intersect(self, adsp1, adsp2):
		ads1 = AdVec()
		ads2 = AdVec()
		ads1.add_vec(adsp1)
		ads2.add_vec(adsp2)
		for ad1 in ads1.data:
			present = False
			for ad2 in ads2.data:
				if(self.same_ads(ad1, ad2)):
					present = True
					break
			if(present):
				self.add(ad1)
				ads2.remove(ad2)
		return self
					
	def ad_weight(self, ad):
		count = 0
		for a in self.data:
			if(self.same_ads(a, ad)):
				count = count+1
		if(W_CHOICE == NUM):
			return count
		elif(W_CHOICE == LOG_NUM):
			if(count == 0):
				return 0
			else:
				return math.log(count)
		else:
			print("Illegal W_CHOICE")
			sys.exit()
		
	def gen_temp_ad_vec(self, ads):		# self is ad_union
		vec = [0]*ads.size()
		i = 0
		j = 0
		for ad in ads.data:
			for lad in self.data:
				if(self.same_ads(ad, lad)):
					vec[i] = self.index(lad)
			i += 1
		return vec
		
	def gen_ad_vec(self, ads):			# self is ad_union
		vec = [0]*self.size()
		for ad in self.data:
			for lad in ads.data:
				if(self.same_ads(ad, lad)):
					vec[self.index(ad)] += 1
		return vec	
		
	def gen_cat_vec(self, ucats):		# self is ads
		vec = [0]*len(ucats)
		for ad in self.data:
			for cat in ucats:
				if(ad.cat == cat):
					vec[ucats.index(cat)] += 1
		return vec
		
class Analyst:
	
#### GUHA'S ANALYSIS ####

# Ad Set Comparison

	def ad_sim(self, ads1, ads2):
		if(SIM_CHOICE == JACCARD):
			return self.jaccard_index(ads1, ads2)
		elif(SIM_CHOICE == COSINE):
			return self.cosine_sim(ads1, ads2)
		else:
			print("Illegal SIM_CHOICE")
			sys.exit()

	def jaccard_index(self, ads1, ads2):
		ad_union = AdVec()
		ad_union.union(ads1, ads2)
		ad_int = AdVec()
		ad_int.tot_intersect(ads1, ads2)
		return (float(ad_int.size())/float(ad_union.size()))
	
	def ad_cosine_sim(self, ads1, ads2):
		ad_union = AdVec()
		ad_union.union(ads1, ads2)
		vec1 = []
		vec2 = []
		for ad in ad_union.data:
			vec1.append(ads1.ad_weight(ad))
			vec2.append(ads2.ad_weight(ad))
		return self.cosine_sim(vec1, vec2)
	
# Vector Operations

	def cosine_sim(self, vec1, vec2):
		return (self.dot_prod(vec1, vec2)/(self.vec_mag(vec1)*self.vec_mag(vec2)))

	def vec_mag(self, vec):
		sum = 0.0
		for i in vec:
			sum = sum + i*i
		return math.sqrt(sum)
	
	def dot_prod(self, vec1, vec2):
		sum = 0.0
		if(len(vec1) != len(vec2)):
			print("Dot product doesnt exist")
			sys.exit()
		for i in range(0, len(vec1)):
			sum = sum + vec1[i]*vec2[i]
		return sum

#### MY ANALYSES ####
	
	def temp_ad_vectors(self, list):
		ad_union = AdVec()
		for ads in list:
			ad_union.union(ads, ad_union)
		ad_union.display(2)
		tav_list = []
		for ads in list:
			tav_list.append(ad_union.gen_temp_ad_vec(ads))
		return tav_list

	def word_vectors(self, list):
		ad_union = AdVec()
		for ads in list:
			ad_union.union(ads, ad_union)
		words = self.ads_to_words(ad_union)
		stemmed_words = self.stem_vec(words)
		word_v = self.unique_words(stemmed_words)
		word_v = self.strip_vec(word_v)
# 		for asd in word_v:
# 			print str(word_v.index(asd))+" "+asd
		wv_list = []
		for ads in list:
			wv_list.append(self.gen_vec(ads, word_v))
		return wv_list
		
	def ad_vectors(self, list):
		ad_union = AdVec()
		for ads in list:
			ad_union.union(ads, ad_union)
		ad_union.display(2)
		av_list = []
		for ads in list:
			av_list.append(ad_union.gen_ad_vec(ads))
		return av_list

	def ad_category(self, list):
		cats = []
		for ads in list:
			cats.extend(ads.get_cat_list())
		ucats = self.unique_words(cats)
		ucats.sort()
		for c in ucats:
			print (str(ucats.index(c))+" "+c)
		ac_list = []
		for ads in list:
			ac_list.append(ads.gen_cat_vec(ucats))
		return ac_list
	
# Word based analysis

		
	def ads_to_words(self, ads):
		line = ""
		for ad in ads.data:
			line = line + " " + ad.title+ " " + ad.body
		return re.split(r'[.(), !?;\-\n]+', line)
	
	def unique_words(self, words):
		unq = []
		for word in words:
			present = False
			for un in unq:
				if (un == word):
					present = True
					break
			if(not present):
				unq.append(word)
		return unq
	
	def stem_vec(self, words):
		for i in range(0, len(words)-1):
			words[i] = stem(words[i]).lower()
		return words
	
	def strip_vec(self, list):
		if(list[0] == ''):
			del list[-len(list)]
		if(list[len(list)-1] == ''):
			del list[-1]
		chars = set('0123456789$&')
		return [x for x in list if not (any((c in chars) for c in x))]
	
	def gen_vec(self, ads, word_v):
		vec = []
		words = self.ads_to_words(ads)
		stemmed_words = self.stem_vec(words)
		words = self.strip_vec(words)
		# print words
		for word in word_v:
			vec.append(words.count(word))	# set weight of the vectors according to some measure, now its simple count
		# print vec
		# print ("Length = "+str(len(vec)))
		# print ("Number of 0s = "+str(sum(x == 0 for x in vec)))
		return vec
			
# Category based analysis

	def read_categories(self, FILE):
		fo = open(FILE, 'r')
		cats = AdVec()
		for line in fo:
			chunks = re.split("\|\|", line)
			ad = Ad({'Title':chunks[0], 'URL': chunks[1], 'Body': chunks[2], 'cat':chunks[3].rstrip()})
			cats.add(ad)
		return cats		

	def set_categories(self, ads, cat):	
		cats = cat.get_cat_list()
		ucats = self.unique_words(cats)
		ucats.sort()
		i = 0
		for ad in ads.data:
			i += 1
			if( not ad.set_from_cat_file(cat)):
				print('\n')
				ad.display()
				print ("\nExisting Categories:")
				print ucats
				while(True):
					entry = raw_input("Enter category: ")
					con = raw_input("Did you enter "+entry+"? [y/n]")
					if con == 'y':
						break
					else:
						print ("Lets try again...")
				ad.add_to_cat_file(entry, CAT_FILE)
				cat.add(ad)
				cats = cat.get_cat_list()
				ucats = self.unique_words(cats)
				ucats.sort()
				print (str(len(ads.data)-i)+" remaining..")

	

if __name__ == "__main__":
	a = Analyst()
	ran1 = []
	ran2 = []
	adv1 = AdVec()
	adv2 = AdVec()
	int = AdVec()
	adv1.read_ads_from_file("ads1.txt", ran1)
	adv2.read_ads_from_file("ads2.txt", ran2)
	
	cat = a.read_categories(CAT_FILE)
	a.set_categories(adv1, cat)
	a.set_categories(adv2, cat)
	
	list = [adv1, adv2]
	
	#obs = np.array(a.ad_category(list))
	obs = np.array(a.temp_ad_vectors(list))
	#obs = np.array(a.ad_vectors(list))
	#obs = np.array(a.word_vectors(list))
		
	print("Unique in ad1= "+ str(adv1.unique().size()))
	#a.print_ads(a.unique_set(ads1), 1)
	print("Unique in ad2= "+ str(adv2.unique().size()))
	#a.print_ads(a.unique_set(ads2), 1)
	print("Unique Intersection= " +str(int.intersect(adv1, adv2).size()))
	#a.print_ads(a.intersect(ads1, ads2), 1)
	print("Total in ad1= "+ str(adv1.size()))
	print("Total in ad2= "+ str(adv2.size()))
	int = AdVec()
	print("Total Intersection= " +str(int.tot_intersect(adv1, adv2).size()))
	#print a.tot_intersect(ads1, ads2)

	
#### Statistical measures
	
# 	chi2, p, dof, ex = stats.chi2_contingency(obs)
# 	print ("Chi-Square = "+str(chi2))
# 	print ("p-value = "+str(p))
# 	print ("Similarity = "+str(a.ad_sim(ads1, ads2)))
# 	
# 	robjects.r.source('gtest.R')
# 	l = [val for pair in zip(obs[0], obs[1]) for val in pair]
# 	v = robjects.IntVector(l)
# 	m = robjects.r['matrix'](v, nrow = 2)
# 	#print(m)
# 	print("Size of ads:: "+str(len(ads1))+" -- "+str(len(ads2)))
# 	gt = robjects.r['g.test']
# 	ex, dof, psi, gval, pval = gt(m)
# 	print ("Psi-value = "+str(psi[0]))
# 	print ("G-value = "+str(gval[0]))
# 	print ("p-value = "+str(pval[0]))
	
#### Octave Bindings
	
# 	oc = Oct2Py()
# 	oc.plot(obs[0], '-o')
	
#### Temporal Plots

	gridLineWidth=0.1
	fig, ax = plt.subplots()
# 	ax.xaxis.grid(True, zorder=0)
# 	ax.yaxis.grid(True, zorder=0)
	#plt.axes(frameon=False)
	plt.xlabel("Ad ID")
	plt.ylabel("Reload #")
	plt.plot(obs[0], ran1, 'r.', alpha=1)
	plt.plot(obs[1], ran2, 'b+', alpha=1)
# 	plt.axhline(y=17, xmin=-1, xmax=100, color = 'k', ls="dotted")
# 	plt.axhline(y=60, xmin=-1, xmax=100, color = 'k', ls="dotted")
# 	plt.axhline(y=119, xmin=-1, xmax=100, color = 'k', ls="dotted")
# 	plt.axhline(y=178, xmin=-1, xmax=100, color = 'k', ls="dotted")
	plt.show()
	
#### Histogram plots
	
# 	pos = np.arange(len(obs[1]))
# 	width = 0.5     # gives histogram aspect to the bar diagram
# 	
# 	#plt.text(160, 100, r'P1-red, P2-green')
# 	plt.bar(pos, obs[0], width, color='r', alpha=0.5)
# 	#plt.show()
# 	plt.bar(pos, obs[1], width, color='g', alpha=0.5)
# 	plt.show()